/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#define BUFSIZE	(8*1024)
static char buf[BUFSIZE];

extern void error(const char *fmt, ...);
void catreg(int);

void
main(int argc, char *argv[])
{
	int i, fd;

	/*
	 * Process each file named on the command line.
	 */
	for (i = 1; i < argc; i++) {
		if ((fd = open(argv[i], O_RDONLY)) < 0) {
			error("cat: cannot open %s", argv[i]);
			continue;
		}
		catreg(fd);
		close(fd);
	}
	exit(0);
}

void
catreg(int fd)
{
	int nr, nw;

	for (;;) {
		/*
		 * Read a chunk of the file.
		 */
		nr = read(fd, buf, BUFSIZE);
		if (nr > 0) {
			/*
			 * Write the chunk just read.  Clear errno
			 * to silence error() on short writes.
			 */
			errno = 0;
			nw = write(1, buf, nr);
			if (nw != nr) {
				if (nw < 0)
					error("cat: write failed");
				else
					error("cat: short write");
				break;
			}
		} else {
			if (nr != 0)
				error("cat: read failed");
			break;
		}
	}
}
