/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <poll.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <memory.h>
#include <tiuser.h>
#include <netconfig.h>
#include <netdir.h>

#define MAXNET	32

struct pollfd pfd[MAXNET];
int nnet;

void getnets(char *);
void addnet(struct netconfig *, struct netbuf *);

void
getnets(char *service)
{
	struct nd_hostserv hs;
	struct netconfig *ncp;
	struct nd_addrlist *nalp;
	struct netbuf *np;
	void *handle;
	int i;

	nnet = 0;
	hs.h_host = HOST_SELF;
	hs.h_serv = service;

	/*
	 * Gain access to /etc/netconfig.
	 */
	if ((handle = setnetconfig()) == NULL)
		return;

	/*
	 * Get all addresses for this host and service.
	 */
	while ((ncp = getnetconfig(handle)) != NULL) {
		/*
		 * Look for connection-oriented networks only.
		 */
		if ((ncp->nc_semantics == NC_TPI_COTS) ||
		  (ncp->nc_semantics == NC_TPI_COTS_ORD)) {
			if (netdir_getbyname(ncp, &hs, &nalp) == 0) {
				/*
				 * Found some matches.  Add as
				 * many entries as will fit.
				 */
				np = nalp->n_addrs;
				for (i = 0; i < nalp->n_cnt; i++, np++)
					if (nnet < MAXNET)
						addnet(ncp, np);

				/*
				 * Free the address list.
				 */
				netdir_free((void *)nalp, ND_ADDRLIST);
			}
		}
	}

	/*
	 * Free the memory allocated by setnetconfig.
	 */
	endnetconfig(handle);
}
void
addnet(struct netconfig *ncp, struct netbuf *np)
{
	struct t_bind req;
	struct t_bind *retp;
	int fd;

	/*
	 * Open the transport endpoint.
	 */
	fd = t_open(ncp->nc_device, O_RDWR, NULL);
	if (fd < 0)
		return;

	/*
	 * Bind the requested address.
	 */
	retp = (struct t_bind *)t_alloc(fd, T_BIND, T_ADDR);
	if (retp == NULL) {
		t_close(fd);
		return;
	}
	req.addr = *np;	/* structure assignment */
	req.qlen = 1;
	if (t_bind(fd, &req, retp) < 0) {
		t_close(fd);
		t_free(retp, T_BIND);
		return;
	}

	/*
	 * Verify the bound address is what we requested.
	 */
	if ((req.addr.len != retp->addr.len) ||
	  (memcmp(req.addr.buf, retp->addr.buf,
	  req.addr.len) != 0)) {
		t_close(fd);
		t_free(retp, T_BIND);
		return;
	}

	/*
	 * Add the transport endpoint to the array of
	 * file descriptors and free the t_bind structure
	 * previously allocated.
	 */
	pfd[nnet].fd = fd;
	pfd[nnet++].events = POLLIN;
	t_free(retp, T_BIND);
}
